/* 
 * VirtualBox Control Service
 * 
 * Copyright © 2010 vu1tur.eu.org
 * 
 * This file is part of VirtualBox Control Service
 * 
 * VirtualBox Control Service is free software; you can redistribute it 
 * and/or modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2 of 
 * the License, or (at your option) any later version.
 * 
 * VirtualBox Control Service is distributed in the hope that it will 
 * be useful, but WITHOUT ANY WARRANTY; without even the implied 
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */
 
#define WINVER 0x0500

#include "windows.h"
#include "winsvc.h"
#include <process.h>
#include <aclapi.h>
#include <stdio.h>
#include <time.h>
#include <io.h>
#include "sdk/include/VirtualBox.h"

#define VBOX_CONTROL_SERVICE_NAME "vboxctrl"

#define SHUTDOWN_ACTION_POWEROFF 0x01
#define SHUTDOWN_ACTION_SAVESTATE 0x02
#define SHUTDOWN_ACTION_POWERBTN 0x03
#define SHUTDOWN_ACTION_SLEEPBTN 0x04
#define SHUTDOWN_ACTION_DEFAULT SHUTDOWN_ACTION_POWEROFF
#define SHUTDOWN_ACTION_DEFAULT_SZ "poweroff"
#define DEFAULT_PRIORITY_CLASS NORMAL_PRIORITY_CLASS
#define LOGLEVEL_DEFAULT 1

#define SVC_SUCCESS 0
#define SVC_RUNNING 1
#define SVC_ERROR 2
#define _EVENT_STOPVM 0x00
#define _EVENT_STOPSVC 0x01
#ifndef SERVICE_ACCEPT_PRESHUTDOWN
#define SERVICE_ACCEPT_PRESHUTDOWN 0x100
#endif
#ifndef SERVICE_CONTROL_PRESHUTDOWN
#define SERVICE_CONTROL_PRESHUTDOWN 0x0000000F
#endif
#ifndef SERVICE_CONFIG_PRESHUTDOWN_INFO
#define SERVICE_CONFIG_PRESHUTDOWN_INFO 7
#endif
#define MAX_WAIT_TIMEOUT 300000

typedef struct _SERVICE_PRESHUTDOWN_INFO {
  DWORD dwPreshutdownTimeout;
} SERVICE_PRESHUTDOWN_INFO, *LPSERVICE_PRESHUTDOWN_INFO;

#define SAFE_RELEASE(x) \
	if (x) { \
	x->Release(); \
	x = NULL; \
	}

void WINAPI ServiceMain(DWORD argc, LPTSTR *argv);
DWORD WINAPI ServiceCtrlHandler(DWORD Opcode, DWORD dwEventType, void* lpEventData, void* lpContext);
int InstallService();
int DeleteService();
void SetServicePermissions(SC_HANDLE hService);
void ServiceThread(void *args);
void ProgressThread(void *args);
void ReportSvcStatus(DWORD dwCurrentState, DWORD dwWin32ExitCode, DWORD dwWaitHint);
BOOL ReadIniFile(BOOL bAll);
void setenv();
void getver();
BSTR alloc_bstr(const char* str);

HRESULT InitializeVirtualBoxInterface();
HRESULT InitializeVirtualBoxInterface(IVirtualBox **ppVirtualBox);
HRESULT OpenVirtualBoxSession(ISession **ppSession);
BOOL GetVirtualBoxMachine(const char* vm, IMachine **machine);
BOOL GetVirtualBoxMachine(IVirtualBox *pVirtualBox, const char* vm, IMachine **machine);
void CloseVirtualBoxInterface();
void CloseVirtualBoxInterface(IVirtualBox **ppVirtualBox);
BOOL StartVirtualBoxMachine(const char* vm);
void StopVirtualBoxMachine(const char* vm);
WAITORTIMERCALLBACK VMThreadWaitCallback(void*);

BOOL GetMachineVRDPEnabled(IVirtualBox *pVirtualBox, ISession *pSession);
LONG GetMachineVRDPPort(IVirtualBox *pVirtualBox, ISession *pSession);
DWORD GetMachineVRDPResolution(IVirtualBox *pVirtualBox, ISession *pSession);

extern char szvmName[255];

void Log(char loglevel, char* format, ...);
void strcrpl(char *str, char c, char r);