/* 
 * VirtualBox Control Service
 * 
 * Copyright © 2010 vu1tur.eu.org
 * 
 * This file is part of VirtualBox Control Service
 * 
 * VirtualBox Control Service is free software; you can redistribute it 
 * and/or modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2 of 
 * the License, or (at your option) any later version.
 * 
 * VirtualBox Control Service is distributed in the hope that it will 
 * be useful, but WITHOUT ANY WARRANTY; without even the implied 
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */
 
#include "utils.h"
#include "vboxctrltray.h"
#include "vboxipcc.h"

int allocatedSubMenus = 0;
int VBoxMachinesCount = -1;


int APIENTRY WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nShowCmd)
{
	HANDLE hMutex = CreateMutex(NULL, TRUE, "14336562-b3f7-490b-a9bd-858dd2c41bbf");
	if (GetLastError() == ERROR_ALREADY_EXISTS)
	{
		MessageBox(NULL, "Another instance of vboxctrltray is already running", "VirtualBox Control Service tray helper", MB_OK);
		return 1;
	}

	if (_Vista_)
	{
		HMODULE htheme = LoadLibraryA("uxtheme.dll");
		if (htheme)
		{
			bufferedPaintInit = (pfnBufferedPaintInit) GetProcAddress(htheme, "BufferedPaintInit");
			bufferedPaintUninit = (pfnBufferedPaintUninit) GetProcAddress(htheme, "BufferedPaintUninit");
			beginBufferedPaint = (pfnBeginBufferedPaint) GetProcAddress(htheme, "BeginBufferedPaint");
			getBufferedPaintBits = (pfnGetBufferedPaintBits) GetProcAddress(htheme, "GetBufferedPaintBits");
			endBufferedPaint = (pfnEndBufferedPaint) GetProcAddress(htheme, "EndBufferedPaint");
		}
	}

	if (bufferedPaintInit)
		bufferedPaintInit();

	ZeroMemory(&StatusHBitmaps, sizeof(HBITMAP)*VMSTATUS_LAST);

	g_hInst = hInstance;
	HWND hwnd = CreateDialog(hInstance, MAKEINTRESOURCE(IDD_DLG), NULL, (DLGPROC)DlgProc);
	if (!hwnd)
		return 1;
		
	MSG msg;
	while (GetMessage(&msg, NULL, 0, 0))
	{
		if (!IsDialogMessage(msg.hwnd, &msg))
		{
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
	}
	
	if (bufferedPaintUninit)
		bufferedPaintUninit();
		
	ReleaseMutex(hMutex);
	return 0;
}

BOOL CreateNotificationIcon(HWND hwnd)
{
	WM_TASKBARCREATED = RegisterWindowMessageA("TaskbarCreated");
	nid.cbSize = sizeof(nid);
	nid.uID = 1;
	nid.hWnd = hwnd;
	nid.uFlags = NIF_ICON | NIF_MESSAGE | NIF_TIP;
	nid.hIcon = (HICON)LoadImage(GetModuleHandle(NULL), MAKEINTRESOURCE(IDI_TRAYICON), IMAGE_ICON, 16, 16, 0);
	strcpy(nid.szTip, "No VirtualBox machines running");
	nid.uCallbackMessage = WMAPP_NOTIFYCALLBACK;
	nid.uVersion = NOTIFYICON_VERSION;

	if (!Shell_NotifyIcon(NIM_ADD, &nid))
		return FALSE;
	Shell_NotifyIcon(NIM_SETVERSION, &nid);
}

void CreateContextMenu(int lastError)
{
	int i;
	MENUITEMINFO mi;
	g_hMenu = CreatePopupMenu();
	
	if (VBoxMachinesCount < 0)
		AppendMenu(g_hMenu, MF_STRING | MF_GRAYED, 0, "No configuration file");
	else if (VBoxMachinesCount == 0)
		AppendMenu(g_hMenu, MF_STRING | MF_GRAYED, 0, "No services configured");
	else {
		g_hSubMenus = (HMENU*)realloc(g_hSubMenus, VBoxMachinesCount*sizeof(HMENU));
		allocatedSubMenus = VBoxMachinesCount;
		for (i = 0; i < VBoxMachinesCount; i++) {
			g_hSubMenus[i] = CreatePopupMenu();
			if (lastError == VBCST_ERROR_SCM_DENIED) {
				AppendMenu(g_hSubMenus[i], MF_STRING | MF_GRAYED, 0, "Service Control Manager access denied");
				VBoxMachines[i].status = VMSTATUS_SVCERROR;
			} else if (lastError == VBCST_ERROR_SVC_DENIED)
				AppendMenu(g_hSubMenus[i], MF_STRING | MF_GRAYED, 0, "Service access denied");
			else if (VBoxMachines[i].status == VMSTATUS_MISSING)
				AppendMenu(g_hSubMenus[i], MF_STRING | MF_GRAYED, 0, "Not configured");
			else if (VBoxMachines[i].status == VMSTATUS_NOTFOUND)
				AppendMenu(g_hSubMenus[i], MF_STRING | MF_GRAYED, 0, "Configuration file not found");
			else if (VBoxMachines[i].status == VMSTATUS_SVCERROR)
				AppendMenu(g_hSubMenus[i], MF_STRING | MF_GRAYED, 0, "Service not found");
			else {
				AppendMenu(g_hSubMenus[i], MF_STRING | (VBoxMachines[i].status != VMSTATUS_STOPPED ? MF_GRAYED : 0), VMMENU_START(i), "Start");
				AppendMenu(g_hSubMenus[i], MF_STRING | (VBoxMachines[i].status != VMSTATUS_RUNNING ? MF_GRAYED : 0), VMMENU_STOP(i), "Stop");
				AppendMenu(g_hSubMenus[i], MF_SEPARATOR, 0, NULL);
				AppendMenu(g_hSubMenus[i], MF_STRING | 
					((VBoxIPCC_GetVRDPEnabled(VBoxMachines[i].displayName) && VBoxMachines[i].status == VMSTATUS_RUNNING) ? 0 : MF_GRAYED), 
					VMMENU_CONSOLE(i), "VRDP console");
				AppendMenu(g_hSubMenus[i], MF_STRING | (_access(VBoxMachines[i].logFile, 0) == 0 ? 0 : MF_GRAYED), VMMENU_SHOWLOG(i), "Show log");
				AppendMenu(g_hSubMenus[i], MF_SEPARATOR, 0, NULL);
				AppendMenu(g_hSubMenus[i], MF_STRING | (VBoxMachines[i].autostart ? MF_CHECKED : 0), VMMENU_AUTO(i), "Autostart");
			}
			AppendMenu(g_hMenu, MF_STRING | MF_POPUP, (UINT_PTR)g_hSubMenus[i], VBoxMachines[i].displayName);
		}
	}
	AppendMenu(g_hMenu, MF_SEPARATOR, 0, NULL);
	AppendMenu(g_hMenu, MF_STRING, IDM_RELOAD, "Reload config");
	AppendMenu(g_hMenu, MF_STRING | (IsAutostarted() ? MF_CHECKED : 0), IDM_AUTOSTART, "Autostart on logon");
	AppendMenu(g_hMenu, MF_SEPARATOR, 0, NULL);
	AppendMenu(g_hMenu, MF_STRING, IDM_EXIT, "Close");
	
	for (i = 0; i < VBoxMachinesCount; i++)
	{
		ZeroMemory(&mi, sizeof(mi));
		mi.cbSize = sizeof(mi);
		mi.fMask = MIIM_BITMAP | MIIM_ID;
		if (_Vista_)
		{
			if (StatusHBitmaps[VBoxMachines[i].status] == NULL) {
				HICON hicon = GetStatusIcon(VBoxMachines[i].status);
				StatusHBitmaps[VBoxMachines[i].status] = (HBITMAP)IconToBitmap(hicon);
				DestroyIcon(hicon);
			}
			mi.hbmpItem = StatusHBitmaps[VBoxMachines[i].status];
		} else {
			mi.hbmpItem = HBMMENU_CALLBACK;
		}
		mi.wID = VBoxMachines[i].status;
		SetMenuItemInfo(g_hMenu, i, TRUE, &mi);
	}
}

void DeleteContextMenu()
{
	int i;
	for (i = allocatedSubMenus; i--;)
		DestroyMenu(g_hSubMenus[i]);
	free(g_hSubMenus);
	g_hSubMenus = NULL;
	DestroyMenu(g_hMenu);
}

void ShowContextMenu(HWND hwnd)
{
	POINT cp;
	GetCursorPos(&cp);
	SetForegroundWindow(hwnd);
	TrackPopupMenu(g_hMenu, TPM_LEFTALIGN | TPM_BOTTOMALIGN, cp.x, cp.y, 0, hwnd, NULL);
}

INT_PTR CALLBACK DlgProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	int i;
	int err;
	SC_HANDLE hService;
	MEASUREITEMSTRUCT *lpmis = NULL;
	DRAWITEMSTRUCT *lpdis = NULL;
	HICON hIcon;

	if (message == WM_TASKBARCREATED) {
		CreateNotificationIcon(hwnd);
		return 0;
	}
		
	switch (message)
	{
		case WM_INITDIALOG:
			err = ReadConfig();
			CreateNotificationIcon(hwnd);
			if (err == VBCST_SUCCESS)
				err = UpdateServices();
			CreateContextMenu(err);
			break;

		case WMAPP_NOTIFYCALLBACK:
			switch (lParam)
			{
				case WM_LBUTTONUP:
				case WM_RBUTTONUP:
					DeleteContextMenu();
					err = UpdateServices();
					CreateContextMenu(err);
					ShowContextMenu(hwnd);
					break;
			}
			break;
			
		case WM_MEASUREITEM:
			lpmis = (MEASUREITEMSTRUCT*)lParam;
			if (lpmis == NULL)
				break;
			lpmis->itemWidth += 3;
			if (lpmis->itemHeight < 16)
				lpmis->itemHeight = 16;
			break;
			
		case WM_DRAWITEM:
			lpdis = (DRAWITEMSTRUCT*)lParam;
			if ((lpdis == NULL)||(lpdis->CtlType != ODT_MENU))
				break;
			hIcon = GetStatusIcon(lpdis->itemID);
			if (hIcon != NULL)
			{
				DrawIconEx(lpdis->hDC, lpdis->rcItem.left - 16, (lpdis->rcItem.top + lpdis->rcItem.bottom - 16)/2, hIcon, 16, 16, 0, NULL, DI_NORMAL);
				DestroyIcon(hIcon);
			}
			break;
			
		case WM_COMMAND:
			switch (LOWORD(wParam))
			{
				case IDM_EXIT:
					DeleteContextMenu();
					CloseTrayHelper();
					PostQuitMessage(0);
					break;
				case IDM_RELOAD:
					err = ReadConfig();
					DeleteContextMenu();
					UpdateServices();
					CreateContextMenu(err);
					break;
				case IDM_AUTOSTART:
					ToggleAutostart();
					break;
			}
			if (LOWORD(wParam) > IDM_FIRSTVM)
			{
				for (i = 0; i < VBoxMachinesCount; i++) 
				{
					if (LOWORD(wParam) == VMMENU_START(i))
					{
						g_hSCM = OpenSCManager(NULL, SERVICES_ACTIVE_DATABASE, SC_MANAGER_CONNECT);
						if (g_hSCM == NULL)
							break;
						hService = OpenService(g_hSCM, VBoxMachines[i].serviceName, GENERIC_READ | SERVICE_START);
						if (hService != NULL)
						{
							StartService(hService, 0, NULL);
							CloseServiceHandle(hService);
						}
						CloseServiceHandle(g_hSCM);
					}
					else if (LOWORD(wParam) == VMMENU_STOP(i))
					{
						g_hSCM = OpenSCManager(NULL, SERVICES_ACTIVE_DATABASE, SC_MANAGER_CONNECT);
						if (g_hSCM == NULL)
							break;
						hService = OpenService(g_hSCM, VBoxMachines[i].serviceName, GENERIC_READ | SERVICE_STOP);
						if (hService != NULL)
						{
							SERVICE_STATUS tmp;
							ControlService(hService, SERVICE_CONTROL_STOP, &tmp);
							CloseServiceHandle(hService);
						}
						CloseServiceHandle(g_hSCM);
					}
					else if (LOWORD(wParam) == VMMENU_AUTO(i))
					{
						g_hSCM = OpenSCManager(NULL, SERVICES_ACTIVE_DATABASE, SC_MANAGER_CONNECT);
						if (g_hSCM == NULL)
							break;
						hService = OpenService(g_hSCM, VBoxMachines[i].serviceName, GENERIC_READ | SERVICE_CHANGE_CONFIG);
						if (hService != NULL)
						{
							ChangeServiceConfig(
								hService, 
								SERVICE_WIN32_OWN_PROCESS | SERVICE_INTERACTIVE_PROCESS, 
								VBoxMachines[i].autostart ? SERVICE_DEMAND_START : SERVICE_AUTO_START, 
								SERVICE_NO_CHANGE,
								NULL, NULL, NULL, NULL, NULL, NULL, NULL);
							CloseServiceHandle(hService);
						}
						CloseServiceHandle(g_hSCM);
					}
					else if (LOWORD(wParam) == VMMENU_SHOWLOG(i))
					{
						SHELLEXECUTEINFO sei = { 0 };
						sei.cbSize = sizeof(SHELLEXECUTEINFO);
						sei.fMask = SEE_MASK_CLASSNAME;
						sei.lpVerb = "open";
						sei.lpFile = VBoxMachines[i].logFile;
						sei.nShow = SW_SHOWNORMAL;
						sei.lpClass = ".txt";
						ShellExecuteEx(&sei);
					}
					else if (LOWORD(wParam) == VMMENU_CONSOLE(i))
					{
						LONG vrdpWidth, vrdpHeight;
						LONG vrdpPort;
						vrdpPort = (LONG) VBoxIPCC_GetVRDPPort(VBoxMachines[i].displayName);
						if (vrdpPort > 0) {
							VBoxIPCC_GetVRDPResolution(VBoxMachines[i].displayName, &vrdpWidth, &vrdpHeight);
							char params[128];
							if ((vrdpWidth > 0) && (vrdpHeight > 0)) 
								sprintf(params, "/v 127.0.0.1:%d /w %d /h %d", vrdpPort, vrdpWidth, vrdpHeight);
							else
								sprintf(params, "/v 127.0.0.1:%d /f", vrdpPort);
							
							SHELLEXECUTEINFO sei = { 0 };
							sei.cbSize = sizeof(SHELLEXECUTEINFO);
							sei.fMask = 0;
							sei.lpVerb = "open";
							sei.lpFile = "mstsc.exe";
							sei.lpParameters = params;
							sei.nShow = SW_SHOWNORMAL;
							ShellExecuteEx(&sei);
						}
					}
				}
			}
			break;
	}
	return 0;
}

int ReadConfig()
{
	GetModuleFileName(NULL, g_szModule, MAX_PATH);
	strcpy(g_szIniFile, g_szModule);
	if (strlen(g_szIniFile) > 4)
		strcpy(g_szIniFile + strlen(g_szIniFile) - 4, ".ini");
	else
		return VBCST_ERROR_NOCONF;
	
	VBoxMachinesCount = 0;
	if (_access(g_szIniFile, 0) == -1)
	{
		VBoxMachinesCount = -1;
		return VBCST_ERROR_NOCONF;
	}
	
	int bufSize = 1024;
	char *buf = (char*) malloc(bufSize);
	if (buf == NULL)
		return VBCST_ERROR_MEMALLOC;
	DWORD bytesRead = GetPrivateProfileSectionNames(buf, bufSize, g_szIniFile);
	while (bytesRead == bufSize - 2) {
		bufSize *= 2;
		if (bufSize > 16777216)
			return VBCST_ERROR_INICORRUPTED;
		buf = (char*)realloc(buf, bufSize);
		if (buf == NULL)
			return VBCST_ERROR_MEMALLOC;
		bytesRead = GetPrivateProfileSectionNames(buf, bufSize, g_szIniFile);
	}

	char *pbuf = buf;
	while (*pbuf != 0) {
		VBoxMachinesCount++;
		VBoxMachines = realloc(VBoxMachines, VBoxMachinesCount * sizeof(VBoxMachineInfo));
		ZeroMemory(&VBoxMachines[VBoxMachinesCount-1], sizeof(VBoxMachineInfo));
		strncpy(VBoxMachines[VBoxMachinesCount-1].name, pbuf, 127);
		GetPrivateProfileString(VBoxMachines[VBoxMachinesCount-1].name, 
			"ConfigFile", "", 
			VBoxMachines[VBoxMachinesCount-1].configFile, MAX_PATH-1, 
			g_szIniFile);
		strcrpl(VBoxMachines[VBoxMachinesCount-1].configFile, '/', '\\');

		if (strlen(VBoxMachines[VBoxMachinesCount-1].configFile) > 4)
		{
			strcpy(VBoxMachines[VBoxMachinesCount-1].logFile, VBoxMachines[VBoxMachinesCount-1].configFile);
			strcpy(VBoxMachines[VBoxMachinesCount-1].logFile + strlen(VBoxMachines[VBoxMachinesCount-1].logFile) - 4, ".log");
		}
		
		strcpy(VBoxMachines[VBoxMachinesCount-1].displayName, VBoxMachines[VBoxMachinesCount-1].name);
		VBoxMachines[VBoxMachinesCount-1].status = VMSTATUS_UNKNOWN;
		
		if (strlen(VBoxMachines[VBoxMachinesCount-1].configFile) == 0)
			VBoxMachines[VBoxMachinesCount-1].status = VMSTATUS_MISSING;
		else if (_access(VBoxMachines[VBoxMachinesCount-1].configFile, 0) == -1)
			VBoxMachines[VBoxMachinesCount-1].status = VMSTATUS_NOTFOUND;
		else {
			GetPrivateProfileString("VBoxCtrl", "VmName", "", 
				VBoxMachines[VBoxMachinesCount-1].displayName, 255, 
				VBoxMachines[VBoxMachinesCount-1].configFile);
			if (strlen(VBoxMachines[VBoxMachinesCount-1].displayName) == 0)
			{
				strcpy(VBoxMachines[VBoxMachinesCount-1].displayName, VBoxMachines[VBoxMachinesCount-1].name);
				VBoxMachines[VBoxMachinesCount-1].status = VMSTATUS_MISSING;
			} else {
				strcpy(VBoxMachines[VBoxMachinesCount-1].serviceName, strrchr(VBoxMachines[VBoxMachinesCount-1].configFile, '\\') + 1);
				VBoxMachines[VBoxMachinesCount-1].serviceName[strlen(VBoxMachines[VBoxMachinesCount-1].serviceName)-4] = '\0';
			}
		}
		pbuf += strlen(pbuf) + 1;
	}
	if (buf != NULL)
		free(buf);
	
	return VBCST_SUCCESS;
}

int UpdateServices()
{
	int i,n;
	int ret = VBCST_SUCCESS;
	SC_HANDLE hService;
	g_hSCM = OpenSCManager(NULL, SERVICES_ACTIVE_DATABASE, SC_MANAGER_CONNECT);
	if (g_hSCM == NULL)
	{
		if (GetLastError() == ERROR_ACCESS_DENIED)
			return VBCST_ERROR_SCM_DENIED;
		return VBCST_ERROR_NOCONF;
	} else {
		for (i = 0; i < VBoxMachinesCount; i++)
		{
			if (VBoxMachines[i].status > VMSTATUS_UNKNOWN && VBoxMachines[i].status != VMSTATUS_SVCERROR)
				continue;
			VBoxMachines[i].vrdpPort = 0;
			
			hService = OpenService(g_hSCM, VBoxMachines[i].serviceName, GENERIC_READ | SERVICE_STOP | SERVICE_START);
			if (hService == NULL) {
				VBoxMachines[i].status = VMSTATUS_SVCERROR;
				if (GetLastError() == ERROR_ACCESS_DENIED)
					ret = VBCST_ERROR_SVC_DENIED;
			}
			else 
			{
				SERVICE_STATUS svcStatus;
				QueryServiceStatus(hService, &svcStatus);
				switch (svcStatus.dwCurrentState)
				{
					case SERVICE_RUNNING:
						VBoxMachines[i].status = VMSTATUS_RUNNING;
						VBoxMachines[i].vrdpPort = GetPrivateProfileInt("VBoxCtrl", "VRDPPort", 0, VBoxMachines[i].configFile);
						break;
					case SERVICE_START_PENDING:
						VBoxMachines[i].status = VMSTATUS_STARTING;
						break;
					case SERVICE_STOP_PENDING:
						VBoxMachines[i].status = VMSTATUS_STOPPING;
						break;
					case SERVICE_STOPPED:
						VBoxMachines[i].status = VMSTATUS_STOPPED;
						break;
					default:
						VBoxMachines[i].status = VMSTATUS_UNKNOWN;
				}

				QUERY_SERVICE_CONFIG *svcConfig = NULL;
				DWORD dwBytesNeeded = 0;
				QueryServiceConfig(hService, NULL, 0, &dwBytesNeeded);
				if (GetLastError() == ERROR_INSUFFICIENT_BUFFER)
				{
					svcConfig = (QUERY_SERVICE_CONFIG*) malloc(dwBytesNeeded);
					if (svcConfig != NULL)
					{
						QueryServiceConfig(hService, svcConfig, dwBytesNeeded, &dwBytesNeeded);
						VBoxMachines[i].autostart = (svcConfig->dwStartType == SERVICE_AUTO_START) ? TRUE : FALSE;
						free(svcConfig);
					}
				}
				
				CloseServiceHandle(hService);
			}
		}

		CloseServiceHandle(g_hSCM);
	}

	if (ret != VBCST_SUCCESS)
		return ret;
		
	n = 0;
	for (i = 0; i < VBoxMachinesCount; i++) 
	{
		if (VBoxMachines[i].status == VMSTATUS_RUNNING)
			n++;
	}
	if (n > 0)
		sprintf(nid.szTip, "%d VirtualBox machine%s running", n, (n > 1) ? "s" : "");
	else
		sprintf(nid.szTip, "No VirtualBox machines running");	

	Shell_NotifyIcon(NIM_MODIFY, &nid);
	
	return ret;
}

void CloseTrayHelper()
{
	int i;
	Shell_NotifyIcon(NIM_DELETE, &nid);
	if (VBoxMachinesCount > 0) {
		free(VBoxMachines);
	}

	for(i = VMSTATUS_LAST; i--;)
		if (StatusHBitmaps[i] != NULL)
			DeleteObject(StatusHBitmaps[i]);
}

BOOL IsAutostarted()
{
	BOOL ret;
	HKEY hKey;
	DWORD buffersize = MAX_PATH;
	char path[MAX_PATH];
	if (RegOpenKeyEx(HKEY_CURRENT_USER, "Software\\Microsoft\\Windows\\CurrentVersion\\Run", 0, KEY_READ, &hKey) == ERROR_SUCCESS)
	{
		ret = (RegQueryValueEx(hKey, "vboxctrltray", NULL, NULL, (BYTE*)&path, &buffersize) == ERROR_SUCCESS);
		RegCloseKey(hKey);
		return ret;
	}
	return FALSE;
}

void ToggleAutostart()
{
	HKEY hKey;
	DWORD buffersize = MAX_PATH;
	char path[MAX_PATH];
	if (RegOpenKeyEx(HKEY_CURRENT_USER, "Software\\Microsoft\\Windows\\CurrentVersion\\Run", 0, KEY_ALL_ACCESS, &hKey) == ERROR_SUCCESS)
	{
		if (RegQueryValueEx(hKey, "vboxctrltray", NULL, NULL, (BYTE*)&path, &buffersize) == ERROR_SUCCESS)
			RegDeleteValue(hKey, "vboxctrltray");
		else {
			sprintf(path, "\"%s\"", g_szModule);
			RegSetValueEx(hKey, "vboxctrltray", 0, REG_SZ, path, strlen(path)+1);
		}
		RegCloseKey(hKey);
	}
}

HICON GetStatusIcon(int status)
{
	switch(status)
	{
		case VMSTATUS_STOPPED:
			return (HICON)LoadImage(g_hInst, MAKEINTRESOURCE(IDI_STOPPED), IMAGE_ICON, 16, 16, 0);
		case VMSTATUS_RUNNING:
			return (HICON)LoadImage(g_hInst, MAKEINTRESOURCE(IDI_RUNNING), IMAGE_ICON, 16, 16, 0);
		case VMSTATUS_STOPPING:
		case VMSTATUS_STARTING:
			return (HICON)LoadImage(g_hInst, MAKEINTRESOURCE(IDI_WORKING), IMAGE_ICON, 16, 16, 0);
		case VMSTATUS_MISSING:
		case VMSTATUS_SVCERROR:
		case VMSTATUS_NOTFOUND:
			return (HICON)LoadImage(g_hInst, MAKEINTRESOURCE(IDI_ERRORICON), IMAGE_ICON, 16, 16, 0);
	}
	return NULL;
}
